package GuitarKey.Gui;

import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.logging.Logger;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import GuitarKey.Utils.ApplicationModes;


/**
 * Class controls the buttons and their actions
 */
public class ButtonMap extends JPanel
{
	// Logger
	private static Logger logger = Logger.getLogger("GuitarKey");

	// space between buttons
	private int BUTTON_OFFSET_X = 20;
	private int BUTTON_OFFSET_Y = 9;
	private int BUTTON_WIDTH = 29;
	private int BUTTON_HEIGHT = 27;
	private int BUTTON_SPACING = 1;
	private int NUMBER_OF_BUTTONS = 3;

	private int BUTTON_MAP_WIDTH = 798;
	private int BUTTON_MAP_HEIGHT = 620;

	private ActionListener actionListener;

	private JLabel infoLabel;
	private HashMap infoMap;

	// Search Items
	private JTextField searchNameField;
	private ScaleButton searchButton;

	private JComboBox scaleListBox;
	private JComboBox positionListBox;

	// Save Scale items
	private ScaleButton saveScaleButton;
	private ScaleButton cancelSaveButton;
	private JLabel scaleNameLabel;
	private JTextField scaleNameField;
	private JLabel scaleRootLabel;
	private JComboBox scaleRootBox;

	// Save Tuning items
	private ScaleButton saveTuningButton;
	private JLabel tuningNameLabel;
	private JTextField tuningNameField;

	// Audio Items
	private TurnButton tempoButton;
	private TurnButton volumeButton;
	private GuitarButtonGroup guitarButtonGroup;

	/**
	 * Constructor
	 */
	public ButtonMap(ActionListener actionListener)
	{
		super();

		logger.info("Creating ButtonMap");

		this.actionListener = actionListener;

		// ButtonMap properties
		setOpaque(false);
		setBounds(0, 0, BUTTON_MAP_WIDTH, BUTTON_MAP_HEIGHT);
		setLayout(null);

		// Buttons
		buildTopButtons();
		buildBottomButtons();

		// Save Interface
		buildSaveInterface();
		hideSaveInterface();

		// Position Interface
		buildPositionInterface();

		// Save Tuning Interface
		hideSaveTuningInterface();

		// Information Label
		buildInfoLabelMap();
		hideInfo();
	}

	/**
	 * Build the button list
	 */
	private void buildTopButtons()
	{
		logger.info("Building the top buttons");

		// Save Button
		ScaleButton saveButton = new ScaleButton("save");
		saveButton.setBounds(BUTTON_OFFSET_X, BUTTON_OFFSET_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
		saveButton.addActionListener(actionListener);

		// Clear Button
		ScaleButton clearButton = new ScaleButton("clear");
		clearButton.setBounds(BUTTON_OFFSET_X+BUTTON_WIDTH+BUTTON_SPACING,
								BUTTON_OFFSET_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
		clearButton.addActionListener(actionListener);

		// Pick notes Button
		ScaleButton pickButton = new ScaleButton("pick");
		pickButton.setBounds(BUTTON_OFFSET_X+(BUTTON_WIDTH+BUTTON_SPACING)*2,
								BUTTON_OFFSET_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
		pickButton.addActionListener(actionListener);

		// Find Button
		ScaleButton findButton = new ScaleButton("find");
		findButton.setBounds(BUTTON_OFFSET_X+(BUTTON_WIDTH+BUTTON_SPACING)*3,
								BUTTON_OFFSET_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
		findButton.addActionListener(actionListener);

		// Search Fields
		searchNameField = new JTextField();
		searchNameField.setBounds(BUTTON_MAP_WIDTH - 260 - 4,
								BUTTON_OFFSET_Y + 2, 200, 22);
		searchNameField.setBackground(ApplicationModes.COLOR_LIGHT_GRAY);
		searchNameField.setForeground(ApplicationModes.COLOR_GRAY);
		searchNameField.setFont(ApplicationModes.FONT_GEORGIA10);
		searchNameField.setSelectionColor(ApplicationModes.COLOR_LIGHT_BLUE);
		searchNameField.setBorder(new LineBorder(ApplicationModes.COLOR_GRAY, 1));
		searchNameField.setToolTipText("Enter a scale name");

		// Find Button
		searchButton = new ScaleButton("search");
		searchButton.setBounds(BUTTON_MAP_WIDTH - 60,
								BUTTON_OFFSET_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
		searchButton.addActionListener(actionListener);

		// Scale List Combo Box
		scaleListBox = new JComboBox();
		scaleListBox.addItem("-- No matching scales --");
		scaleListBox.setBounds(BUTTON_OFFSET_X+(BUTTON_WIDTH+BUTTON_SPACING)*4+20,
						BUTTON_OFFSET_Y + 2, 200, 22);
		scaleListBox.setBackground(ApplicationModes.COLOR_LIGHT_GRAY);
		scaleListBox.setForeground(ApplicationModes.COLOR_GRAY);
		scaleListBox.setBorder(new LineBorder(ApplicationModes.COLOR_GRAY, 1));
		scaleListBox.setFont(ApplicationModes.FONT_GEORGIA10);
		scaleListBox.addActionListener(actionListener);
		scaleListBox.setActionCommand("Scale Selected");

		// Add buttons
		add(saveButton);
		add(clearButton);
		add(findButton);
		add(pickButton);
		add(searchNameField);
		add(searchButton);
		add(scaleListBox);
	}

	/**
	 * Build the bottom button list
	 */
	private void buildBottomButtons()
	{
		logger.info("Building the bottom buttons");

		// Tempo Button
		tempoButton = new TurnButton("Tempo", 100, 400);
		tempoButton.setLocation(BUTTON_OFFSET_X+10, BUTTON_MAP_HEIGHT - 106);

		// Volume Button
		volumeButton = new TurnButton("Volume", 0, 100);
		volumeButton.setLocation(BUTTON_OFFSET_X+110, BUTTON_MAP_HEIGHT - 106);

		// Play Buttons
		ScaleButton playScaleButton = new ScaleButton("play_scale");
		playScaleButton.setBounds(BUTTON_OFFSET_X+550,
								BUTTON_MAP_HEIGHT - 103, 35, 36);
		playScaleButton.addActionListener(actionListener);

		ScaleButton playChordButton = new ScaleButton("play_chord");
		playChordButton.setBounds(BUTTON_OFFSET_X+590,
								BUTTON_MAP_HEIGHT - 103, 35, 36);
		playChordButton.addActionListener(actionListener);

		// Guitar Buttons
		guitarButtonGroup = new GuitarButtonGroup();
		guitarButtonGroup.setLocation(BUTTON_MAP_WIDTH-124, BUTTON_MAP_HEIGHT - 98);

		add(tempoButton);
		add(volumeButton);
		add(playScaleButton);
		add(playChordButton);

		for (int i=0; i<GuitarButtonGroup.NUMBER_OF_GUIT_BUT; i++)
		{
			add(GuitarButtonGroup.buttons[i]);
		}
	}

	/**
	 * Build the position drop down interface
	 */
	private void buildPositionInterface()
	{
		// Position List Combo Box
		positionListBox = new JComboBox();

		// Add Positions
		positionListBox.addItem("I");
		positionListBox.addItem("II");
		positionListBox.addItem("III");
		positionListBox.addItem("IV");
		positionListBox.addItem("V");
		positionListBox.addItem("VI");
		positionListBox.addItem("VII");

		positionListBox.setToolTipText("Select Filter Position");

		// Set Properties
		positionListBox.setBounds(BUTTON_OFFSET_X+(BUTTON_WIDTH+BUTTON_SPACING)*4+20+220,
						BUTTON_OFFSET_Y + 2, 46, 22);
		positionListBox.setBackground(ApplicationModes.COLOR_LIGHT_GRAY);
		positionListBox.setForeground(ApplicationModes.COLOR_GRAY);
		positionListBox.setBorder(new LineBorder(ApplicationModes.COLOR_GRAY, 1));
		positionListBox.setFont(ApplicationModes.FONT_GEORGIA10);
		positionListBox.addActionListener(actionListener);
		positionListBox.setActionCommand("Set Position");

		add(positionListBox);
	}

	/**
	 * Build the interface to save a new scale
	 */
	private void buildSaveInterface()
	{
		logger.info("Building the save scale interface");

		// Save Scale Button
		saveScaleButton = new ScaleButton("save_scale");
		saveScaleButton.setBounds(620, ApplicationModes.TOP_GUIT_LOCATION - 20, 37, 22);
		saveScaleButton.addActionListener(actionListener);

		// Cancel Save
		cancelSaveButton = new ScaleButton("cancel_scale");
		cancelSaveButton.setBounds(665, ApplicationModes.TOP_GUIT_LOCATION - 20, 47, 22);
		cancelSaveButton.addActionListener(actionListener);

		// Scale Name Label
		scaleNameLabel = new JLabel("Enter Scale Name");
		scaleNameLabel.setBounds(405, ApplicationModes.TOP_GUIT_LOCATION - 40, 200, 20);
		scaleNameLabel.setForeground(ApplicationModes.COLOR_GRAY_BORDER);
		scaleNameLabel.setFont(ApplicationModes.FONT_GEORGIA11);

		// Scale Name Text Field
		scaleNameField = new JTextField();
		scaleNameField.setBounds(405, ApplicationModes.TOP_GUIT_LOCATION - 20, 200, 22);
		scaleNameField.setBackground(ApplicationModes.COLOR_LIGHT_GRAY);
		scaleNameField.setForeground(ApplicationModes.COLOR_GRAY);
		scaleNameField.setFont(ApplicationModes.FONT_GEORGIA10);
		scaleNameField.setSelectionColor(ApplicationModes.COLOR_LIGHT_BLUE);
		scaleNameField.setBorder(new LineBorder(ApplicationModes.COLOR_GRAY, 1));

		// Root Name Label
		scaleRootLabel = new JLabel("Select Scale Root");
		scaleRootLabel.setBounds(295, ApplicationModes.TOP_GUIT_LOCATION - 40, 100, 20);
		scaleRootLabel.setForeground(ApplicationModes.COLOR_GRAY_BORDER);
		scaleRootLabel.setFont(ApplicationModes.FONT_GEORGIA11);

		// Scale Root Combo Box
		scaleRootBox = new JComboBox();
		for (int i=0; i<ApplicationModes.NUMBER_OF_NOTES_IN_OCTAVE; i++)
		{
			scaleRootBox.addItem(ApplicationModes.Note_Names[i]);
		}
		scaleRootBox.setBounds(295, ApplicationModes.TOP_GUIT_LOCATION - 20, 100, 22);
		scaleRootBox.setBackground(ApplicationModes.COLOR_LIGHT_GRAY);
		scaleRootBox.setForeground(ApplicationModes.COLOR_GRAY);
		scaleRootBox.setBorder(new LineBorder(ApplicationModes.COLOR_GRAY, 1));
		scaleRootBox.setFont(ApplicationModes.FONT_GEORGIA10);

		// Save Tuning Button
		saveTuningButton = new ScaleButton("save_tuning");
		saveTuningButton.setBounds(620, ApplicationModes.TOP_GUIT_LOCATION - 20, 37, 22);
		saveTuningButton.addActionListener(actionListener);

		// Tuning Name Label
		tuningNameLabel = new JLabel("Enter Tuning Name");
		tuningNameLabel.setBounds(405, ApplicationModes.TOP_GUIT_LOCATION - 40, 200, 20);
		tuningNameLabel.setForeground(ApplicationModes.COLOR_GRAY_BORDER);
		tuningNameLabel.setFont(ApplicationModes.FONT_GEORGIA11);

		// Tuning Name Text Field
		tuningNameField = new JTextField();
		tuningNameField.setBounds(405, ApplicationModes.TOP_GUIT_LOCATION - 20, 200, 22);
		tuningNameField.setBackground(ApplicationModes.COLOR_LIGHT_GRAY);
		tuningNameField.setForeground(ApplicationModes.COLOR_GRAY);
		tuningNameField.setFont(ApplicationModes.FONT_GEORGIA10);
		tuningNameField.setSelectionColor(ApplicationModes.COLOR_LIGHT_BLUE);
		tuningNameField.setBorder(new LineBorder(ApplicationModes.COLOR_GRAY, 1));

		add(saveScaleButton);
		add(cancelSaveButton);
		add(scaleNameLabel);
		add(scaleNameField);
		add(scaleRootLabel);
		add(scaleRootBox);
		add(saveTuningButton);
		add(tuningNameLabel);
		add(tuningNameField);
	}

	/**
	 * Method builds the information label map
	 */
	private void buildInfoLabelMap()
	{
		logger.info("Building information messages");

		infoMap = new HashMap(32);

		// Information Messages
		infoMap.put("No Notes", "* No notes have been clicked");
		infoMap.put("Scale Saved", "Scale has been saved");
		infoMap.put("Tuning Saved", "Tuning has been saved");
		infoMap.put("No Name", "* Must enter a name in the field");
		infoMap.put("No Match", "* No matching scales found");
		infoMap.put("Pick Mode", "Select notes to specify chord to play");
		infoMap.put("No Help", "* Could not open help, try opening from help folder");

		infoLabel = new JLabel();
		infoLabel.setBounds(38, ApplicationModes.TOP_GUIT_LOCATION - 65, 600, 20);
		infoLabel.setForeground(ApplicationModes.COLOR_BLUE);
		infoLabel.setBackground(null);
		infoLabel.setFont(ApplicationModes.FONT_GEORGIA11);

		add(infoLabel);
	}

	/**
	 * Method hides the save scale items
	 */
	public void hideSaveInterface()
	{
		saveScaleButton.setVisible(false);
		cancelSaveButton.setVisible(false);
		scaleNameLabel.setVisible(false);
		scaleNameField.setVisible(false);
		scaleRootLabel.setVisible(false);
		scaleRootBox.setVisible(false);
	}

	/**
	 * Method displays the save scale items
	 */
	public void showSaveInterface()
	{
		saveScaleButton.setVisible(true);
		cancelSaveButton.setVisible(true);
		scaleNameLabel.setVisible(true);
		scaleNameField.setVisible(true);
		scaleRootLabel.setVisible(true);
		scaleRootBox.setVisible(true);
	}

	/**
	 * Method hides the save tuning items
	 */
	public void hideSaveTuningInterface()
	{
		saveTuningButton.setVisible(false);
		cancelSaveButton.setVisible(false);
		tuningNameLabel.setVisible(false);
		tuningNameField.setVisible(false);
	}

	/**
	 * Method displays the save tuning items
	 */
	public void showSaveTuningInterface()
	{
		saveTuningButton.setVisible(true);
		cancelSaveButton.setVisible(true);
		tuningNameLabel.setVisible(true);
		tuningNameField.setVisible(true);
	}

	/**
	 * Method shows the information label
	 */
	public void showInfo(String inKey)
	{
		if (infoMap.containsKey(inKey))
		{
			infoLabel.setText((String)infoMap.get(inKey));
			infoLabel.setVisible(true);
		}
	}

	/**
	 * Method hides the information label
	 */
	public void hideInfo()
	{
		infoLabel.setVisible(false);
	}

	/**
	 * Method sets the list of scale names from a search
	 */
	public void setScaleList(ArrayList inScaleList)
	{
		logger.info("Populating scale list combo");

		if (!inScaleList.isEmpty())
		{
			scaleListBox.removeAllItems();
			scaleListBox.addItem("-- Select a scale -- (" + inScaleList.size() + ")");

			Iterator itrScaleList = inScaleList.iterator();
			while (itrScaleList.hasNext())
			{
				scaleListBox.addItem((String)itrScaleList.next());
			}
		}
		else
		{
			showInfo("No Match");
		}
	}

	/**
	 * Method gets the root note selected in the dropdown
	 */
	public String getScaleRoot()
	{
		return scaleRootBox.getSelectedItem().toString();
	}

	/**
	 * Method gets the scale name entered in the box
	 */
	public String getScaleName()
	{
		StringBuffer resultBuffer = new StringBuffer();
		String scaleName = scaleNameField.getText();

		// Don't allow ridiculously long names
		if (scaleName.length() > 40)
		{
			scaleName = scaleName.substring(0, 40);
		}

		// Ignore pound sign = false so remove from string
		scaleName = escapeCharacters(scaleName, false);

		return scaleName;
	}

	/**
	 * Method gets the tuning name entered in the box
	 */
	public String getTuningName()
	{
		StringBuffer resultBuffer = new StringBuffer();
		String tuningName = tuningNameField.getText();

		// Don't allow ridiculously long names
		if (tuningName.length() > 40)
		{
			tuningName = tuningName.substring(0, 40);
		}

		// Ignore pound sign = false so remove from string
		tuningName = escapeCharacters(tuningName, false);

		return tuningName;
	}

	/**
	 * Method gets the scale name search string entered
	 */
	public String getSearchName()
	{
		StringBuffer resultBuffer = new StringBuffer();
		String searchName = searchNameField.getText();

		// Don't allow ridiculously long names
		if (searchName.length() > 50)
		{
			searchName = searchName.substring(0, 50);
		}

		// Ignore pound sign = true so leave in search string
		searchName = escapeCharacters(searchName, true);

		return searchName;
	}

	/**
	 * Method gets the scale name of the selected scale
	 */
	public String getSelectedScaleName()
	{
		return ((String)scaleListBox.getSelectedItem());
	}

	/**
	 * Method gets the selected position
	 */
	public String getSelectedPosition()
	{
		return ((String)positionListBox.getSelectedItem());
	}

	/**
	 * Method gets the tempo from the tempo button
	 */
	public int getTempo()
	{
		return tempoButton.getValue();
	}

	/**
	 * Method gets the volume from the volume button
	 */
	public int getVolume()
	{
		return volumeButton.getValue();
	}

	/**
	 * Method gets the guitar midi number of selected guitar (acoustic, electric, jazz, etc)
	 */
	public int getGuitarType()
	{
		return guitarButtonGroup.getGuitarSelected();
	}

	/**
	 * Method escapes the characters of an input string to remove characters who have misbehaved
	 */
	private String escapeCharacters(String inString, boolean ignorePound)
	{
		StringBuffer resultBuffer = new StringBuffer();

		// Strip characters that will cause the file name to crap out
		char[] characters = inString.toCharArray();
		char character;

		for (int i=0; i<characters.length; i++)
		{
			character = characters[i];

			if (character == '.')
			{
		        // do nothing
		    }
			else if (character == '\\')
			{
				// do nothing
			}
			else if (character == '?')
			{
				// do nothing
			}
			else if (character == '*')
			{
				// do nothing
			}
			else if (character == '+')
			{
				// do nothing
			}
			else if (character == '&')
			{
				// do nothing
			}
			else if (character == ':')
			{
				// do nothing
			}
			else if (character == '{')
			{
				// do nothing
			}
			else if (character == '}')
			{
				// do nothing
			}
			else if (character == '[')
			{
				// do nothing
			}
			else if (character == ']')
			{
				// do nothing
			}
			else if (character == '(')
			{
				// do nothing
			}
			else if (character == ')')
			{
				// do nothing
			}
			else if (character == '^')
			{
				// do nothing
			}
			else if (character == '$')
			{
				// do nothing
			}
			else if (character == '<')
			{
				// do nothing
			}
			else if (character == '>')
			{
				// do nothing
			}
			else if (character == '/')
			{
				// do nothing
			}
			else if (character == '"')
			{
				// do nothing
			}
			else if (character == '|')
			{
				// do nothing
			}
			else if (character == '#' && !ignorePound)
			{
				// do nothing
			}
			else
			{
				resultBuffer.append(character);
			}
		}

		return resultBuffer.toString();
	}
}
