package GuitarKey.Gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JButton;

import GuitarKey.Sound.MidiPlayer;

/**
 * Class holds the button group of guitars for selecting guitars
 */
public class GuitarButtonGroup
{
	// Logger
	private static Logger logger = Logger.getLogger("GuitarKey");

	private int BUTTON_WIDTH = 26;
	private int BUTTON_HEIGHT = 26;

	private JButton acousticButton;
	private JButton electricButton;
	private JButton jazzButton;

	public static JButton[] buttons;

	private int guitarType;

	private int BUTTON_SPACING = 5;

	public static final int NUMBER_OF_GUIT_BUT = 3;


	/**
	 * Constructor
	 */
	public GuitarButtonGroup()
	{
		logger.info("Creating GuitarButtonGroup");

		buildButtonGroup();

		buttons = new JButton[NUMBER_OF_GUIT_BUT];

		buttons[0] = acousticButton;
		buttons[1] = jazzButton;
		buttons[2] = electricButton;

		guitarType = MidiPlayer.ACOUSTIC_GUITAR;
	}

	/**
	 * Build the button group
	 */
	private void buildButtonGroup()
	{
		acousticButton = buildButton("acoustic");
		electricButton = buildButton("electric");
		jazzButton = buildButton("jazz");

		GuitarButtonListener listener = new GuitarButtonListener();
		acousticButton.addActionListener(listener);
		jazzButton.addActionListener(listener);
		electricButton.addActionListener(listener);

		acousticButton.setSelected(true);
	}

	/**
	 * Method returns the midi number for the guitar selected
	 */
	public int getGuitarSelected()
	{
		if (acousticButton.isSelected())
		{
			guitarType = MidiPlayer.ACOUSTIC_GUITAR;
		}
		else if (jazzButton.isSelected())
		{
			guitarType = MidiPlayer.JAZZ_GUITAR;
		}
		else if (electricButton.isSelected())
		{
			guitarType = MidiPlayer.ELECTRIC_GUITAR;
		}

		return guitarType;
	}

	/**
	 * Set the button group location
	 */
	public void setLocation(int inX, int inY)
	{
		// Set the buttons locations
		acousticButton.setBounds(inX, inY, BUTTON_WIDTH, BUTTON_HEIGHT);
		jazzButton.setBounds(inX + BUTTON_WIDTH + BUTTON_SPACING, inY, BUTTON_WIDTH, BUTTON_HEIGHT);
		electricButton.setBounds(inX + (BUTTON_WIDTH + BUTTON_SPACING)*2, inY, BUTTON_WIDTH, BUTTON_HEIGHT);
	}

	/**
	 * Build a guitar button
	 */
	private JButton buildButton(String inName)
	{
		JButton result = new JButton();

		result.setFocusable(false);
		result.setBorder(null);
		result.setContentAreaFilled(false);
		result.setIcon(new ImageIcon(getClass().getClassLoader().getResource("img/menu/" + inName + ".png")));
		result.setSelectedIcon(new ImageIcon(getClass().getClassLoader().getResource("img/menu/" + inName + "_click.png")));
		result.setActionCommand(inName);

		return result;
	}

	// Listener for button clicks
	class GuitarButtonListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			String action = e.getActionCommand();

			if (action.equals("acoustic"))
			{
				logger.info("Selected acoustic guitar");
				acousticButton.setSelected(true);
				jazzButton.setSelected(false);
				electricButton.setSelected(false);
			}
			else if (action.equals("jazz"))
			{
				logger.info("Selected jazz guitar");
				jazzButton.setSelected(true);
				acousticButton.setSelected(false);
				electricButton.setSelected(false);
			}
			else if (action.equals("electric"))
			{
				logger.info("Selected electric guitar");
				electricButton.setSelected(true);
				acousticButton.setSelected(false);
				jazzButton.setSelected(false);
			}
		}
	}

}
