package GuitarKey.Gui;

import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import GuitarKey.Filters.ScaleFilter;
import GuitarKey.Utils.ApplicationModes;

/**
 * Class builds the menu bar
 */
public class GuitarMenuBar extends JMenuBar
{
	// Logger
	private static Logger logger = Logger.getLogger("GuitarKey");

	// Menus
	JMenu menuFile;
	JMenu editFile;
	JMenu tuneMenu;
	JMenu filterMenu;
	JMenu menuHelp;
	JMenu loadMenu;
	JMenu fileFind;

	// Edit
	JMenuItem editFileClear;
	JMenuItem editBottomClear;
	JMenuItem editReset;
  JMenuItem editChangeOrientation;

	// File
	JMenuItem menuFileSave;
	JMenuItem menuFileSaveTune;
	JMenuItem menuFileExit;

	// File - Find allowable off notes
	JMenuItem fileFindNone;
	JMenuItem fileFindOne;
	JMenuItem fileFindTwo;

	// Filter
	JMenuItem menuFilterNone;
	JMenuItem menuFilter3Note;
	JMenuItem menuFilterBox;

	// Help
	JMenuItem menuHelpUse;

	// Menu Icons
	private ImageIcon saveIcon;
	private ImageIcon clearIcon;
	private ImageIcon searchIcon;
	private ImageIcon checkIcon;
	private ImageIcon noIcon;

	private HashMap keyRootMap;

	private ActionListener actionListener;

	/**
	 * Constructor
	 */
	public GuitarMenuBar(ActionListener actionListener)
	{
		super();

		logger.info("Creating GuitarMenuBar");

		this.actionListener = actionListener;

		// Map holds key root menus
		keyRootMap = new HashMap(ApplicationModes.NUMBER_OF_NOTES_IN_OCTAVE);

		loadImages();

		buildMenu();
	}

	/**
	 * Method loads the menu images
	 */
	private void loadImages()
	{
		ClassLoader cl = getClass().getClassLoader();

		saveIcon = new ImageIcon(cl.getResource("img/menu/save_icon.png"));
		clearIcon = new ImageIcon(cl.getResource("img/menu/clear_icon.png"));
		searchIcon = new ImageIcon(cl.getResource("img/menu/search_icon.png"));
		checkIcon = new ImageIcon(cl.getResource("img/menu/check_icon.png"));
		noIcon = new ImageIcon(cl.getResource("img/menu/no_icon.png"));
	}

	/**
	 * Method builds the menu
	 */
	private void buildMenu()
	{
		// Menus
		menuFile = new JMenu();
        editFile = new JMenu();
        tuneMenu = new JMenu();
        filterMenu = new JMenu();
        menuHelp = new JMenu();
        loadMenu = new JMenu();
        fileFind = new JMenu();

        // Menu Items
        // Edit
        editFileClear = new JMenuItem();
        editBottomClear = new JMenuItem();
        editReset = new JMenuItem();
        editChangeOrientation = new JMenuItem();
        fileFindNone = new JMenuItem();
        fileFindOne = new JMenuItem();
        fileFindTwo = new JMenuItem();

        // File
        menuFileSave = new JMenuItem();
        menuFileSaveTune = new JMenuItem();
        menuFileExit = new JMenuItem();

        // Filter
        menuFilterNone = new JMenuItem();
		menuFilter3Note = new JMenuItem();
		menuFilterBox = new JMenuItem();

        // Help
        menuHelpUse = new JMenuItem();

        // File
        menuFile.setLabel("File");

        // Edit - Find matching scales
        fileFind.setLabel("Find Matching Scales");
        fileFind.setIcon(searchIcon);

        // File - Find Scale allow no off notes
        fileFindNone.setLabel("Allow No Off Notes");
        fileFindNone.setActionCommand("Find Scales 0");
        fileFindNone.addActionListener(actionListener);

        // File - Find Scale allow one off note
        fileFindOne.setLabel("Allow One Off Note");
        fileFindOne.setActionCommand("Find Scales 1");
        fileFindOne.addActionListener(actionListener);

        // File - Find Scale allow two off notes
        fileFindTwo.setLabel("Allow Two Off Notes");
        fileFindTwo.setActionCommand("Find Scales 2");
        fileFindTwo.addActionListener(actionListener);

        // File - Save Scale
        menuFileSave.setLabel("Save Scale");
        menuFileSave.setIcon(saveIcon);
        menuFileSave.setActionCommand("Save");
        menuFileSave.addActionListener(actionListener);

        // File - Save Tuning
        menuFileSaveTune.setLabel("Save Tuning");
        menuFileSaveTune.setActionCommand("Save Tuning");
        menuFileSaveTune.addActionListener(actionListener);

        // File - Exit
        menuFileExit.setLabel("Exit");
        menuFileExit.setActionCommand("Exit");
        menuFileExit.addActionListener(actionListener);

        // Edit
        editFile.setLabel("Edit");

        // Edit - Clear upper Fretboard
        editFileClear.setLabel("Clear Upper Fretboard");
        editFileClear.setIcon(clearIcon);
        editFileClear.setActionCommand("Clear Scale");
        editFileClear.addActionListener(actionListener);

        // Edit - Clear lower Fretboard
        editBottomClear.setLabel("Clear Lower Fretboard");
        editBottomClear.setActionCommand("Clear Display");
        editBottomClear.addActionListener(actionListener);

        // Edit - Reset Guitars to standard tuning
        editReset.setLabel("Tune Guitars Standard");
        editReset.setActionCommand("Tune Standard");
        editReset.addActionListener(actionListener);

        // Orientation
        editChangeOrientation.setLabel("Change Fretboard Orientation");
        editChangeOrientation.setActionCommand("Change Orientation");
        editChangeOrientation.addActionListener(actionListener);
        
        // Tune
        tuneMenu.setLabel("Tune");

        // Filter
        filterMenu.setLabel("Filter");

        // Filter - No Filter
        menuFilterNone.setLabel("No Filter");
        menuFilterNone.setIcon(checkIcon);
        menuFilterNone.setActionCommand("No Filter");
        menuFilterNone.addActionListener(actionListener);

        // Filter - 3 Note Per String Filter
        menuFilter3Note.setLabel("3 Note Per String");
        menuFilter3Note.setIcon(noIcon);
        menuFilter3Note.setActionCommand("3 Note Filter");
        menuFilter3Note.addActionListener(actionListener);

        // Filter - Box Filter
        menuFilterBox.setLabel("Box Filter");
        menuFilterBox.setIcon(noIcon);
        menuFilterBox.setActionCommand("Box Filter");
        menuFilterBox.addActionListener(actionListener);

        // Help
        menuHelp.setLabel("Help");

        // Help - Using GSA
        menuHelpUse.setLabel("Using Guitar Scale Assistant");
        menuHelpUse.setActionCommand("Show Help");
        menuHelpUse.addActionListener(actionListener);

        loadMenu.setLabel("View Scale");

        for (int i=0; i<ApplicationModes.NUMBER_OF_NOTES_IN_OCTAVE; i++)
        {
        	// Menus key = root note
        	JMenu keyMenu = new JMenu();
        	keyMenu.setLabel(ApplicationModes.Note_Names[i]);
        	keyRootMap.put(ApplicationModes.Note_Names[i], keyMenu);
        	loadMenu.add(keyMenu);
        }

        fileFind.add(fileFindNone);
        fileFind.add(fileFindOne);
        fileFind.add(fileFindTwo);

        menuFile.add(loadMenu);
        menuFile.addSeparator();
        menuFile.add(fileFind);
        menuFile.addSeparator();
        menuFile.add(menuFileSave);
        menuFile.add(menuFileSaveTune);
        menuFile.addSeparator();
        menuFile.add(menuFileExit);

        editFile.add(editFileClear);
        editFile.add(editBottomClear);
        editFile.addSeparator();
        editFile.add(editReset);
        editFile.addSeparator();
        editFile.add(editChangeOrientation);

        filterMenu.add(menuFilterNone);
        filterMenu.add(menuFilter3Note);
        filterMenu.add(menuFilterBox);

        menuHelp.add(menuHelpUse);

        add(menuFile);
        add(editFile);
        add(tuneMenu);
        add(filterMenu);
        add(menuHelp);
	}

	/**
	 * Method sets the check box icons for the filter menu
	 */
	public void setFilterMenu()
	{
		int filterMode = ApplicationModes.filterMode;

		switch (filterMode)
		{
			case ScaleFilter.NO_FILTER:
				menuFilterNone.setIcon(checkIcon);
				menuFilter3Note.setIcon(noIcon);
				menuFilterBox.setIcon(noIcon);
				break;

			case ScaleFilter.THREE_NOTE_FILTER:
				menuFilterNone.setIcon(noIcon);
				menuFilter3Note.setIcon(checkIcon);
				menuFilterBox.setIcon(noIcon);
				break;

			case ScaleFilter.BOX_FILTER:
				menuFilterNone.setIcon(noIcon);
				menuFilter3Note.setIcon(noIcon);
				menuFilterBox.setIcon(checkIcon);
				break;
		}
	}

	/**
	 * Method adds a new scale name to the appropriate menu
	 */
	public void addScaleName(String inScaleName)
	{
		// Get root from name
		int spaceIdx = inScaleName.indexOf(" ");
		String keyRoot = inScaleName.substring(0, spaceIdx);

		// Build Scale Name Menu Item
		JMenuItem tmpMenuItem = new JMenuItem();
    	tmpMenuItem.setLabel(inScaleName);
    	tmpMenuItem.setActionCommand(inScaleName);
    	tmpMenuItem.addActionListener(actionListener);

    	// Add Menu Item to appropriate menu
    	if (keyRootMap.containsKey(keyRoot))
    	{
    		JMenu rootMenu = (JMenu)keyRootMap.get(keyRoot);

    		int itemCount = rootMenu.getItemCount();

    		if (itemCount > 24)
    		{
    			// Add a more... menu to cut down on size
    			JMenu moreMenu = new JMenu("more...");
    			rootMenu.add(moreMenu);
    			keyRootMap.put(keyRoot, moreMenu);
    			rootMenu = moreMenu;
    			itemCount = 0;
    		}
    		if (itemCount > 0)
    		{
    			String lastScaleName = rootMenu.getItem(itemCount-1).getLabel();

    			int index = lastScaleName.indexOf(" ");
    			if (index > 0)
    			{
    				String groupStr = lastScaleName.substring(index+1, index+2);
    				String groupStr2 = inScaleName.substring(index+1, index+2);

    				// Add seperators by first letter of scale name (after root)
    				if (!groupStr.equals(groupStr2))
    				{
    					rootMenu.addSeparator();
    				}
    			}
    		}

    		rootMenu.add(tmpMenuItem);
    	}
	}

	/**
	 * Method adds a new tuning name to the tuning menu
	 */
	public void addTuningName(String inTuningName)
	{
		JMenuItem tmpMenuItem = new JMenuItem();
		tmpMenuItem.setLabel(inTuningName);
    	tmpMenuItem.setActionCommand("Tune -" + inTuningName);
    	tmpMenuItem.addActionListener(actionListener);

    	tuneMenu.add(tmpMenuItem);
	}
}
