package GuitarKey.Gui;

import java.util.HashMap;

import javax.swing.ImageIcon;
import javax.swing.JButton;

/**
 * Class holds button objects
 */
public class ScaleButton extends JButton
{
	private String buttonName;

	private ImageIcon buttonNormalImage;
	private ImageIcon buttonOverImage;
	private ImageIcon buttonClickImage;

	private HashMap helpMap;
	private HashMap actionMap;


	/**
	 * Constructor
	 */
	public ScaleButton(String inName)
	{
		super();

		buttonName = inName;

		loadImages();

		// Available Buttons Help Text
		helpMap = new HashMap(32);
		helpMap.put("save", "Save this scale");
		helpMap.put("clear", "Clear the upper fretboard");
		helpMap.put("find", "Find matching scales");
		helpMap.put("pick", "Select notes for chord");
		helpMap.put("search", "Search scales by name");
		helpMap.put("play_scale", "Play the scale only");
		helpMap.put("play_chord", "Play scale and chord together");

		// Available Buttons Action Commands
		actionMap = new HashMap(32);
		actionMap.put("save", "Save");
		actionMap.put("clear", "Clear Scale");
		actionMap.put("find", "Find Scales");
		actionMap.put("pick", "Pick Note Mode");
		actionMap.put("search", "Search By Name");
		actionMap.put("play_scale", "Play Scale");
		actionMap.put("play_chord", "Play Chord");

		actionMap.put("save_scale", "Save Scale");
		actionMap.put("save_tuning", "Save New Tuning");
		actionMap.put("cancel_scale", "Cancel Save");

		configureButton();
	}

	/**
	 * Method loads the buttons images for regular and over states
	 */
	private void loadImages()
	{
		ClassLoader cl = getClass().getClassLoader();

		buttonNormalImage = new ImageIcon(cl.getResource("img/menu/" + buttonName + ".png"));
		buttonOverImage = new ImageIcon(cl.getResource("img/menu/" + buttonName + "_over.png"));
		buttonClickImage = new ImageIcon(cl.getResource("img/menu/" + buttonName + "_click.png"));
	}

	/**
	 *
	 */
	private void configureButton()
	{
		// button display properties
        setFocusable(false);
        setBorder(null);
        setContentAreaFilled(false);
        setIcon(buttonNormalImage);
        setRolloverIcon(buttonOverImage);
        setPressedIcon(buttonClickImage);

        // Set Action Command from HashMap
        if (actionMap.containsKey(buttonName))
        {
        	setActionCommand((String)actionMap.get(buttonName));
        }

        // Set Help Text from HashMap
        if (helpMap.containsKey(buttonName))
        {
        	setToolTipText((String)helpMap.get(buttonName));
        }
	}

}
