package GuitarKey.Instruments;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Logger;

import GuitarKey.Utils.ShiftUnit;

/**
 * Class provides contains the tuning map for a guitar w/ all tuners
 */
class TuningMap
{
	// Logger
	private static Logger logger = Logger.getLogger("GuitarKey");

	private int x;
	private int y;

	private int HEIGHT = 138;
	private int WIDTH = 51;

	ArrayList tuners;

	/**
	 * Constructor
	 */
	public TuningMap()
	{
		logger.info("Creating Tuner Map");

		buildTunerList();
	}

	/**
	 * Method builds the tuner list and adds the tuners
	 */
	private void buildTunerList()
	{
		tuners = new ArrayList();

		// Build the tuners for the six strings
		for (int i=1; i<=6; i++)
		{
			tuners.add(new Tuner(i));
		}
	}


	/**
	 * Method sets the tuner map x location
	 */
	public void setX(int inX)
	{
		x = inX;

		Iterator itrTuners = tuners.iterator();
    	while (itrTuners.hasNext())
    	{
    		((Tuner)itrTuners.next()).setX(x);
    	}
	}

	/**
	 * Method sets the tuner map y location
	 */
	public void setY(int inY)
	{
		y = inY;

		Iterator itrTuners = tuners.iterator();
    	while (itrTuners.hasNext())
    	{
    		Tuner tmpTuner = (Tuner)itrTuners.next();
    		int tunerY = y + (tmpTuner.getHeight() * (tmpTuner.getStringNumber()-1));
    		tmpTuner.setY(tunerY);
    	}
	}

	/**
	 * Method gets the array list of tuners
	 */
	public ArrayList getTuners()
	{
		return tuners;
	}


	/**
	 * Process the click point and return the shift unit (string, shift)
	 */
    public ShiftUnit processClick(Point clickPoint)
    {
    	ShiftUnit shiftUnit = null;

    	Iterator itrTuners = tuners.iterator();
    	while (itrTuners.hasNext())
    	{
    		Tuner tempTuner = (Tuner)itrTuners.next();
    		if (tempTuner.containsPoint(clickPoint))
    		{
    			shiftUnit = tempTuner.processClick(clickPoint);
    		}
    	}

    	return shiftUnit;
    }

    /**
	 * Method returns the tuners to standard tuning
	 * returns array list of shift units to reset clickable fretboard
	 */
    public ArrayList resetToStandard()
    {
    	ArrayList shiftUnits = new ArrayList();

    	Iterator itrTuners = tuners.iterator();
    	while (itrTuners.hasNext())
    	{
    		Tuner tempTuner = (Tuner)itrTuners.next();
    		shiftUnits.add(tempTuner.getStandardTuneShift());
    		tempTuner.tuneStandard();
    	}

    	return shiftUnits;
    }

    /**
     * Method returns the array list of shift units
     * Opposite direction of reset
     */
    public ArrayList getTuningShifts()
    {
    	ArrayList shiftUnits = new ArrayList();

    	Iterator itrTuners = tuners.iterator();
    	while (itrTuners.hasNext())
    	{
    		Tuner tempTuner = (Tuner)itrTuners.next();
    		shiftUnits.add(tempTuner.getCumulativeShift());
    	}

    	return shiftUnits;
    }

    /**
     *	Method tunes the tuning map for specified ShiftUnit
     */
    public void tune(ShiftUnit inShiftUnit)
    {
    	Iterator itrTuners = tuners.iterator();
    	while (itrTuners.hasNext())
    	{
    		Tuner tempTuner = (Tuner)itrTuners.next();

    		int tunerNum = tempTuner.getStringNumber();
    		int shiftNum = inShiftUnit.getStringNum();

    		// ShiftUnit string number matches tuner string number, tune this tuner
    		if (tunerNum == shiftNum)
    		{
    			int shift = inShiftUnit.getShift();
    			if (shift < 0)
    			{
    				// Tune Up
    				int shiftPos = Math.abs(shift);

    				for (int i=0; i<shiftPos; i++)
    				{
    					tempTuner.tuneUp();
    				}
    			}
    			else if (shift > 0)
    			{
    				// Tune Down
    				for (int i=0; i<shift; i++)
    				{
    					tempTuner.tuneDown();
    				}
    			}
    		}
    	}
    }

    /**
	 * Method tests whether tuner contains the clicked point
	 */
    public boolean containsPoint(Point inPoint)
    {
    	boolean containsPoint = false;

		int pY = (int)inPoint.getY();
		int pX = (int)inPoint.getX();

    	if ((pY > y) && (pY < (y+HEIGHT))
    	 && (pX > x) && (pX < (x+WIDTH)))
    	{
    		containsPoint = true;
    	}

    	return containsPoint;
    }
}
