package GuitarKey.Utils;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.FileInputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Class controls the different application modes and background
 */
public class ApplicationModes
{
	// Logger
	private static Logger logger = Logger.getLogger("GuitarKey");

	// Application Modes
	public static final int FIND_SCALE = 0;
	public static final int CREATE_SCALE = 1;
	public static final int SHIFT_SCALE = 2;

	private int mode = FIND_SCALE;

	// Guitar Properties
	public static final int TOP_GUIT_LOCATION = 125;
	public static final int MIDDLE_GUIT_LOCATION = 200;
	public static final int BOTTOM_GUIT_LOCATION = 310;

	public static final int NUMBER_OF_FRETS = 24; //including 0 and beyond last = 22
	public static final int NUMBER_OF_ACTUAL_FRETS = 22;

	// Guitar X Offset Location
	public static final int LEFT_SHIFT = 17;

	// Note Properties
	public static String[] Note_Names;
	public static String[] Note_Lower_Case_Names;
	public static String[] Note_Upper_Case_Names;
	public static int[] Standard_Tuning_Index;
	public static final int NUMBER_OF_AVAILABLE_NOTES = 48;
	public static final int NUMBER_OF_NOTES_IN_OCTAVE = 12;

	// Font
	public static Font FONT_UBER;
	public static Font FONT_GEORGIA10;
	public static Font FONT_GEORGIA11;
	public static Font FONT_DIGI14;

	// Colors
	public static Color COLOR_RED;
	public static Color COLOR_GRAY;
	public static Color COLOR_GRAY_BORDER;
	public static Color COLOR_LIGHT_GRAY;
	public static Color COLOR_LIGHT_BLUE;
	public static Color COLOR_BLUE;
	public static Color COLOR_YELLOW;

	// Picking note mode (for chords)
	public static boolean pickMode = false;

	// Filtering mode
	public static int filterMode = 0;


	/**
	 * Constructor
	 */
	public ApplicationModes()
	{
		logger.info("Creating ApplicationModes");

		setApplicationMode(FIND_SCALE);

		initializeFont();

		buildNoteProperties();

		COLOR_RED = new Color(60, 25, 25);
		COLOR_GRAY = new Color(37, 36, 36);
		COLOR_GRAY_BORDER = new Color(150, 160, 165);
		COLOR_LIGHT_GRAY = new Color(197, 200, 201);
		COLOR_LIGHT_BLUE = new Color(212, 234, 242);
		COLOR_BLUE = new Color(20, 100, 160);
		COLOR_YELLOW = new Color(205, 211, 167);

	}

	// Method initializes a Uberholme Condensed font
	private void initializeFont()
	{
		try
		{
			logger.info("Loading Fonts");

			// Georgia
			FONT_GEORGIA10 = new Font("Georgia", Font.PLAIN, 10);

			FONT_GEORGIA11 = new Font("Georgia", Font.PLAIN, 11);

			// Uberholme
			File file = new File("fonts/uberc.ttf");
			FileInputStream fis = new FileInputStream(file);
			Font font = Font.createFont(Font.TRUETYPE_FONT, fis);

			FONT_UBER = font.deriveFont(20.0F);

			// Digital Upright
			file = new File("fonts/Digirtu.ttf");
			fis = new FileInputStream(file);
			font = Font.createFont(Font.TRUETYPE_FONT, fis);

			FONT_DIGI14 = font.deriveFont(14.0F);

		}
		catch (Exception ex)
		{
			logger.log(Level.SEVERE, "Error loading fonts", ex);
		}
	}

	// Method builds the note properties
	private void buildNoteProperties()
	{
		logger.info("Loading note and tuner index arrays");

		// Build the Note Names arrays
		Note_Lower_Case_Names = new String[]{"e3", "f3", "fs3", "g3", "gs3", "a3", "as3", "b3", "c4",
					"cs4", "d4", "ds4", "e4", "f4", "fs4", "g4", "gs4", "a4", "as4", "b4", "c5",
					"cs5", "d5", "ds5", "e5", "f5", "fs5", "g5", "gs5", "a5", "as5", "b5", "c6",
					"cs6", "d6", "ds6", "e6", "f6", "fs6", "g6", "gs6", "a6", "as6", "b6", "c7",
					"cs7", "d7", "ds7"};

		Note_Upper_Case_Names = new String[]{"E", "F", "F#", "G", "G#", "A", "A#", "B", "C", "C#", "D", "D#",
					"E", "F", "F#", "G", "G#", "A", "A#", "B", "C", "C#", "D", "D#",
					"E", "F", "F#", "G", "G#", "A", "A#", "B", "C", "C#", "D", "D#",
					"E", "F", "F#", "G", "G#", "A", "A#", "B", "C", "C#", "D", "D#"};

		Note_Names = new String[]{"E", "F", "F#", "G", "G#", "A", "A#", "B", "C", "C#", "D", "D#"};

		// Standard tuning [E4, A4, D5, G5, B5, E6]
		Standard_Tuning_Index = new int[]{12, 17, 22, 27, 31, 36};
	}

	/**
	 * Toggle the chord building mode
	 */
	public static void togglePickMode()
	{
		pickMode = !pickMode;
	}

	/**
	 * Set the filtering mode
	 */
	public static void setFilterMode(int inMode)
	{
		filterMode = inMode;
	}

	/**
	 * Set the application mode
	 */
	public void setApplicationMode(int inMode)
	{
		mode = inMode;
	}

	/**
	 * Get the application mode
	 */
	public int getMode()
	{
		return mode;
	}

}
