package GuitarKey.Utils;

import java.io.File;
import java.lang.reflect.Method;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Class controls the launching of the help files in a browser
 */
public class HelpLauncher
{
	// Logger
	private static Logger logger = Logger.getLogger("GuitarKey");

	private String strOSName = null;

	// Show help main page
    private File helpFile;
    private String helpURL;

	/**
	 * Constructor
	 */
	public HelpLauncher()
	{
		logger.info("Creating ApplicationModes");

		strOSName = System.getProperty("os.name");

		// Get the help file and absolute path
		try
		{
			helpFile = new File("help/index.html");
			helpURL = helpFile.getAbsolutePath();
		}
		catch (Exception ex)
		{
			logger.log(Level.SEVERE, "Could not find help file index", ex);
		}
	}

	/**
	 * Display a url in the clients default browser
	 */
    public boolean showHelp()
	{
		boolean browserAvailable = true;

		try
		{
		 	if (strOSName.startsWith("Mac OS"))
		 	{
		 		Class fileMgr = Class.forName("com.apple.eio.FileManager");
		 		Method openURL = fileMgr.getDeclaredMethod("openURL", new Class[] {String.class});
		 		openURL.invoke(null, new Object[] {helpURL});
		 	}
		 	else if (strOSName.startsWith("Windows"))
		 	{
		 		Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + helpURL);
		 	}
		 	//assume Unix or Linux
		 	else
		 	{
		 		String[] browsers = { "firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape" };
		 		String browser = null;
		 		for (int count = 0; count < browsers.length && browser == null; count++)
		 		{
		 			if (Runtime.getRuntime().exec(new String[] {"which", browsers[count]}).waitFor() == 0)
		 			{
		 				browser = browsers[count];
		 			}
		 			if (browser == null)
		 			{
		 				browserAvailable = false;

		 				throw new Exception("Could not find web browser");
		 			}
		 			else
		 			{
		 				Runtime.getRuntime().exec(new String[] {browser, helpURL});
		 			}

		 		}
			}
		}
		catch (Exception  ex)
		{
			logger.log(Level.SEVERE, "Error showing help files", ex);

			return false;
		}

		return browserAvailable;
	}
}
