package GuitarKey.Utils;

import java.util.ArrayList;
import java.util.Iterator;

/*
 * Class holds a scale object
 */
public class Scale
{
	private ArrayList scaleLocations;
	private String scaleName;
	private String scaleRootNote;
	private String scaleFullName;

	/**
	 * Scale Object constructor
	 */
	public Scale(String scaleRoot, String scaleName)
	{
		this.scaleRootNote = scaleRoot;
		this.scaleName = scaleName;
		scaleFullName = scaleRootNote + " " + scaleName;
		scaleLocations = new ArrayList();
	}

	/**
	 * Add a scale location (string, fret) to the scale
	 */
	public void addScaleLocation(int stringNum, int fretNum)
	{
		// Make sure scale location is not already there
		Iterator itrscaleLocations = scaleLocations.iterator();
		boolean add = true;
		while (itrscaleLocations.hasNext())
		{
			ScaleLocation tmpLoc = (ScaleLocation)itrscaleLocations.next();
			if (tmpLoc.getStringLocation() == stringNum &&
					tmpLoc.getFretLocation() == fretNum)
			{
				add = false;
				if (ApplicationModes.pickMode)
				{
					tmpLoc.setSelected();
				}
			}
		}

		if (add)
		{
			ScaleLocation scaleLoc = new ScaleLocation(stringNum, fretNum);
			if (ApplicationModes.pickMode)
			{
				scaleLoc.setSelected();
			}
			scaleLocations.add(scaleLoc);
		}
	}

	/**
	 * Add a scale location to the scale
	 */
	public void addScaleLocation(ScaleLocation inScaleLoc)
	{
		// Make sure scale location is not already there
		Iterator itrscaleLocations = scaleLocations.iterator();
		boolean add = true;
		while (itrscaleLocations.hasNext())
		{
			ScaleLocation tmpLoc = (ScaleLocation)itrscaleLocations.next();
			if (tmpLoc.getStringLocation() == inScaleLoc.getStringLocation() &&
					tmpLoc.getFretLocation() == inScaleLoc.getFretLocation())
			{
				add = false;
				if (ApplicationModes.pickMode)
				{
					tmpLoc.setSelected();
				}
			}
		}

		if (add)
		{
			if (ApplicationModes.pickMode)
			{
				inScaleLoc.setSelected();
			}
			scaleLocations.add(inScaleLoc);
		}
	}

	/*
	 * Remove a scale location from the scale
	 */
	public void removeScaleLocation(int stringNum, int fretNum)
	{
		Iterator itrscaleLocations = scaleLocations.iterator();
		while (itrscaleLocations.hasNext())
		{
			ScaleLocation tmpLoc = (ScaleLocation)itrscaleLocations.next();
			if (tmpLoc.getStringLocation() == stringNum &&
					tmpLoc.getFretLocation() == fretNum)
			{
				itrscaleLocations.remove();
			}
		}
	}

	/*
	 * Remove a scale location from the scale
	 */
	public void removeScaleLocation(ScaleLocation inScaleLoc)
	{
		Iterator itrscaleLocations = scaleLocations.iterator();
		while (itrscaleLocations.hasNext())
		{
			ScaleLocation tmpLoc = (ScaleLocation)itrscaleLocations.next();
			if (tmpLoc.getStringLocation() == inScaleLoc.getStringLocation() &&
					tmpLoc.getFretLocation() == inScaleLoc.getFretLocation())
			{
				itrscaleLocations.remove();
			}
		}
	}

	/**
	 * Method returns whether scale contains scale location
	 */
	public boolean contains(ScaleLocation inScaleLoc)
	{
		boolean exists = false;

		// See if scale location is already there
		Iterator itrscaleLocations = scaleLocations.iterator();
		while (itrscaleLocations.hasNext())
		{
			ScaleLocation tmpLoc = (ScaleLocation)itrscaleLocations.next();
			if (tmpLoc.getStringLocation() == inScaleLoc.getStringLocation() &&
					tmpLoc.getFretLocation() == inScaleLoc.getFretLocation())
			{
				exists = true;
			}
		}

		return exists;
	}

	/**
	 * Method returns whether scale contains note
	 */
	public boolean contains(String inNoteName)
	{
		boolean exists = false;

		ArrayList noteNames = getNoteNameList();

		if (noteNames.contains(inNoteName))
		{
			exists = true;
		}

		return exists;
	}

	/**
	 * Get the scale root note
	 */
	public String getScaleRoot()
	{
		return scaleRootNote;
	}

	/**
	 * Sets the scale root note
	 */
	public void setScaleRoot(String inRoot)
	{
		scaleRootNote = inRoot;

		scaleFullName = scaleRootNote + " " + scaleName;
	}

	/**
	 * Get the scale name
	 */
	public String getScaleName()
	{
		return scaleName;
	}

	/**
	 * Sets the scale name
	 */
	public void setScaleName(String inScaleName)
	{
		scaleName = inScaleName;

		scaleFullName = scaleRootNote + " " + scaleName;
	}

	/**
	 * Get the scales full name
	 */
	public String getScaleFullName()
	{
		return scaleFullName;
	}

	/**
	 * Get the scale locations
	 */
	public ArrayList getScaleLocations()
	{
		return scaleLocations;
	}

	/**
	 * Method sorts the scale locations by string, fret
	 */
	public void sortScaleLocations()
	{
		int stringNum1;
		int stringNum2;
		int tmpStringNum;
		int fretNum1;
		int fretNum2;
		int tmpFretNum;

		int size = scaleLocations.size();

		ScaleLocation tempScaleLoc;

		Object[] scaleLocs = scaleLocations.toArray();

		// Sort
		for (int i=0; i<size-1; i++)
		{
			stringNum1 = ((ScaleLocation)scaleLocs[i]).getStringLocation();
			fretNum1 = ((ScaleLocation)scaleLocs[i]).getFretLocation();

			for (int j=i; j<size; j++)
			{
				stringNum2 = ((ScaleLocation)scaleLocs[j]).getStringLocation();
				fretNum2 = ((ScaleLocation)scaleLocs[j]).getFretLocation();

				if (stringNum2 < stringNum1)
				{
					// Swap Scale Locations
					tempScaleLoc = (ScaleLocation)scaleLocs[i];
					scaleLocs[i] = scaleLocs[j];
					scaleLocs[j] = tempScaleLoc;

					tmpStringNum = stringNum1;
					stringNum1 = stringNum2;
					stringNum2 = tmpStringNum;

					tmpFretNum = fretNum1;
					fretNum1 = fretNum2;
					fretNum2 = tmpFretNum;
				}
				else if ((stringNum2 == stringNum1) && (fretNum2 < fretNum1))
				{
					// Swap Scale Locations
					tempScaleLoc = (ScaleLocation)scaleLocs[i];
					scaleLocs[i] = scaleLocs[j];
					scaleLocs[j] = tempScaleLoc;

					tmpStringNum = stringNum1;
					stringNum1 = stringNum2;
					stringNum2 = tmpStringNum;

					tmpFretNum = fretNum1;
					fretNum1 = fretNum2;
					fretNum2 = tmpFretNum;
				}
			}
		}

		scaleLocations.clear();
		for (int n=0; n<size; n++)
		{
			scaleLocations.add((ScaleLocation)scaleLocs[n]);
		}
	}

	/**
	 * Get the size (number of scale locations) in this scale
	 */
	public int size()
	{
		return scaleLocations.size();
	}

	/**
	 * Get an array of note names in scale
	 */
	public String[] getNoteNameArray()
	{
		String[] result = null;

		ArrayList noteNames = getNoteNameList();

		result = new String[noteNames.size()];
		noteNames.toArray(result);

		return result;
	}

	/**
	 * Get the list of note names in scale
	 */
	public ArrayList getNoteNameList()
	{
		ArrayList result = new ArrayList();

		Iterator itrscaleLocations = scaleLocations.iterator();
		while (itrscaleLocations.hasNext())
		{
			ScaleLocation tmpLoc = (ScaleLocation)itrscaleLocations.next();
			String noteName = tmpLoc.getNoteName();
			if (!result.contains(noteName))
			{
				result.add(noteName);
			}
		}

		return result;
	}

	/**
	 * Get a clone of the scale
	 */
	public Scale clone()
	{
		Scale scale = new Scale(new String(scaleRootNote), new String(scaleName));
		int fret;
		int string;
		boolean isRoot;
		String name;

		Iterator itrscaleLocations = scaleLocations.iterator();
		while (itrscaleLocations.hasNext())
		{
			ScaleLocation tmpLoc = (ScaleLocation)itrscaleLocations.next();
			fret = tmpLoc.getFretLocation();
			string = tmpLoc.getStringLocation();
			isRoot = tmpLoc.isRoot();
			name = tmpLoc.getNoteName();

			ScaleLocation tmpScaleLoc = new ScaleLocation(string, fret);
			tmpScaleLoc.setRoot(isRoot);
			if (name != null)
			{
				tmpScaleLoc.setNoteName(new String(name));
			}

			scale.addScaleLocation(tmpScaleLoc);
		}

		return scale;
	}

}
