package GuitarKey.Utils;

/*
 * Class Holds Fret and String numbers for scale location
 */
public class ScaleLocation
{
	private int fretLoc = 0;
	private int initialFret = 0;
	private int stringLoc = 0;

	private boolean root = false;
	private boolean aboveOctave = false;

	// Note Selection
	private boolean selected = false;

	// Note alpha (used by filters)
	private boolean applyAlpha = false;

	// Note name
	private String noteName = null;

	// Constructor
	public ScaleLocation(int inStringLoc, int inFretLoc)
	{
		if (inFretLoc >= ApplicationModes.NUMBER_OF_NOTES_IN_OCTAVE)
		{
			inFretLoc = inFretLoc - ApplicationModes.NUMBER_OF_NOTES_IN_OCTAVE;
			aboveOctave = true;
		}

		this.stringLoc = inStringLoc;
		this.fretLoc = inFretLoc;
		initialFret = fretLoc;
	}

	// Return the string number
	public int getStringLocation()
	{
		return stringLoc;
	}

	// Return the fret number
	public int getFretLocation()
	{
		return fretLoc;
	}

	// Set if note is root or not
	public void setRoot(boolean inRoot)
	{
		root = inRoot;
	}

	// Get if scale location is root or not
	public boolean isRoot()
	{
		return root;
	}

	// Set if note is above octave (used for clickable only)
	public void setAboveOctave(boolean inAboveOct)
	{
		aboveOctave = inAboveOct;
	}

	// Get if note is above ocatve (used for clickable fretboard only)
	public boolean isAboveOctave()
	{
		return aboveOctave;
	}

	// Method returns a copy of this scale location set one octave higher
	// Used for drawing purposes only
	public ScaleLocation getAboveOctave()
	{
		ScaleLocation result = new ScaleLocation(stringLoc, fretLoc + ApplicationModes.NUMBER_OF_NOTES_IN_OCTAVE);
		result.setRoot(root);

		return result;
	}

	/**
	 * Toggle whether the scale location is selected
	 * Used for chord building
	 */
	public void setSelected()
	{
		selected = !selected;
	}

	/**
	 * Get if note has been selected for chord or not
	 */
	public boolean isSelected()
	{
		return selected;
	}

	/**
	 * Set whether the alpha is set for the note or not
	 * Used for filters
	 */
	public void setAlpha(boolean inApplyAlpha)
	{
		applyAlpha = inApplyAlpha;
	}

	/**
	 * Get if alpha is applied
	 */
	public boolean alphaApplied()
	{
		return applyAlpha;
	}

	/**
	 * Set the note's name
	 */
	public void setNoteName(String inNoteName)
	{
		noteName = inNoteName;
	}

	/**
	 * Get the note's name
	 */
	public String getNoteName()
	{
		return noteName;
	}

	/**
	 * Method increases the fret location (from string tuned up)
	 */
	public void increase(int offset)
	{
		for (int i=0; i<offset; i++)
		{
			fretLoc++;

			if (fretLoc >= ApplicationModes.NUMBER_OF_NOTES_IN_OCTAVE)
			{
				fretLoc = fretLoc - ApplicationModes.NUMBER_OF_NOTES_IN_OCTAVE;
			}
		}
	}

	/**
	 * Method decreases the fret location (from string tuned down)
	 */
	public void decrease(int offset)
	{
		if (offset < 0) offset = -offset;

		for (int i=0; i<offset; i++)
		{
			fretLoc--;

			if (fretLoc < 0)
			{
				fretLoc = fretLoc + ApplicationModes.NUMBER_OF_NOTES_IN_OCTAVE;
			}
		}
	}

	/**
	 * Method returns this scale location to its original fret (standard tuning)
	 */
	public void tuneStandard()
	{
		fretLoc = initialFret;
	}
}
